#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

#define VGA_START_ADDRESS 0x00800000
#define ADC_START         0x00c00000
#define DAC_START         0x00c00400  
#define DELAY_ADDRESS     0x00c00800
#define SIZE              0x00000100
#define IN_START          0x00c00900

/* the main _interrupt_handler (writen by Xilinx):
1. solve the CPU system issues : saving / restoring context, etc. 
2. sees which interrupts are pending, and for each of them:
2a. acknowledges the interrupt
2b. calls a "user" device handler; their addresses are stores in a table

Our user device handlers are in audio_handler.c and uart_handler.c
I put them there not to keep code organized, but to be able to load them in the LBM
*/

// defined in uart_handler.c
extern volatile int uart_no_interrupts;
extern void uart_handler(void *callback);

// defined in audio_handler.c
//extern Xuint32 *audio_dac_buf, *audio_adc_buf;
extern volatile int audio_dac_bcnt_w, audio_dac_bcnt_r;
extern void audio_handler(void *callback);
//extern int count;
extern int dac_rdptr, dac_wrptr, dac_size, adc_rdptr, adc_wrptr, adc_size;
int data, haveData;
int in[8];



// INTERRUPT  SETUP

// using interrupts at this low level is not trivial
// somebody may find some logic in the following sequence

void setup_interrupts()
{

  // reset the interrupt controller

  // this disables the INTC irq output, so the INTC and CPU are "disconnected"
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  // disable all interrupt sources
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0); // disable all

  // ack all sources, to be sure none of them is pending due to circuit wakeup
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff); // ack all sources


  // initialize the drivers and interrupt handlers
  // here we are also supposed to reset the uart etc. TO DO

  // AUDIO AK4565 DRIVER SETUP

//  XIntc_InterruptVectorTable[XPAR_INTC_AK4565_INTERRUPT_INTR].Handler=audio_handler;
  XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_AK4565_INTERRUPT_INTR, audio_handler, NULL);


  audio_dac_bcnt_w=audio_dac_bcnt_r=0;

  // UART DRIVER & INTERRUPT SETUP


//  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler=uart_handler;
//XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_MYUART_INTERRUPT_INTR, uart_handler, NULL);


  // DONT't forget to initialize your driver data structures !!
  uart_no_interrupts = 0;

  // enable the interrupts

  // enable the CPU interrupts
   microblaze_enable_interrupts();

  // enable the INTC irq output 
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR); // master enable

  // enable the INTC to receive interrupts from the AK4565 and UART
  //  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, 
  //		    XPAR_AK4565_INTERRUPT_MASK | XPAR_MYUART_INTERRUPT_MASK);
  
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AK4565_INTERRUPT_MASK);

  // enable the UART to generate interrupts
  // the audio driver always generate interrupts
  //XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
 
}


// we prepare a square wave for audio output

/*
Xuint16 tri_wave[1024];
void make_triangle_wave()
{
  int i;
  for(i=0;i<512;i++) tri_wave[i]=tri_wave[1023-i]=(i-256)<<7;
}

// this functions ilustrates the user code
// it does 2 things:
// - plays a syrene sound
// - echoes all chars received from the uart

void make_some_noise()
{
  int phasel, phaser, dir, freq,i;
  char c;

  // clear_screen();

  make_triangle_wave();

  phasel=phaser=0; dir=0; freq=10000;
  while(1){

    if (audio_dac_bcnt_w - audio_dac_bcnt_r < 400)
      for(i=0;i<400;i++){
	audio_dac_buf[audio_dac_bcnt_w++ & 0x3FF] = tri_wave[(phaser>>11) & 0x3ff] +
	  (tri_wave[(phasel>>11) & 0x3FF] << 16);

	phasel+= 40000 - freq;
	phaser+=freq;

	if(dir){freq++; if(freq>30000) dir=0;} else
	  {freq--; if(freq<10000) dir=1;}
      }


    putnum(uart_no_interrupts); print("\n");
    
  }

}

*/


main()
{

  int i,a;


  print("Hello world!\n\r");  
  
  dac_rdptr=0;
  dac_wrptr=0;


  for(a=0; a<640*480;a++)
    XIo_Out8(VGA_START_ADDRESS+a, 0);

  // Ok, the code is 6 times faster if cached
//  microblaze_enable_icache();

  for(i=0;i<100;i++)
    XIo_Out8(VGA_START_ADDRESS+i+i*640, 0xff);

  setup_interrupts();

  for(i=0;i<100;i++)
    XIo_Out8(VGA_START_ADDRESS+100+i+i*640, 0xff);

  // USER CODE 
  //  print("Hello interrupts\n\r");

  //make_some_noise();

   for(i=0;i<100;i++)
     XIo_Out8(VGA_START_ADDRESS+200+i+i*640, 0xff);


   in[0] = 0;
   in[1] = 0;
   in[2] = 0;
   in[3] = 0;
   in[4] = -1;
   in[5] = -1;
   in[6] = -1;
   in[7] = -1;
   

     //put input into SRAM
     dac_wrptr=0;
     for(i=0;i<8;i++)
     {
       print("Writing ");
       putnum(in[i]);
       print(" to DAC\n\r");
       XIo_Out32(IN_START+dac_wrptr,in[i]);//write left channel
       dac_wrptr+=4;
     }
       
 
}//end main
