#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

#include "ether_reg.h"

#define XPAR_AK4565_INTERRUPT_MASK 0X000001
#define XPAR_INTC_AK4565_INTERRUPT_INTR 0
#define VGA_START_ADDRESS     0x00800000
#define SIZE                  0x00000050  //160 bytes
#define ADC_START             0x00800000
#define DAC_START             0x00800102
#define DELAY_START1          0x00800500
#define DELAY_START2          0x00802000
#define DELAY_SIZE2           0x00001500
#define DELAY_SIZE1           0x00000800
#define ETH_SEND_SIZE         0x00000050
#define DAC_SIZE              0x00000140
#define SEND_START            0x00806000

/* the main _interrupt_handler (writen by Xilinx):
1. solve the CPU system issues : saving / restoring context, etc.
2. sees which interrupts are pending, and for each of them:
2a. acknowledges the interrupt
2b. calls a "user" device handler; their addresses are stores in a table

Our user device handlers are in audio_handler.c and uart_handler.c
I put them there not to keep code organized, but to be able to load them in the LBM
*/

// defined in uart_handler.c
extern volatile int uart_no_interrupts;
extern void uart_handler(void *callback);

// defined in audio_handler.c
//extern Xuint16 audio_adc_buf[], audio_dac_buf[];
extern volatile int audio_dac_bcnt_w, audio_dac_bcnt_r;
extern void audio_handler(void *callback);

extern int dac_rdptr,dac_wrptr,dac_length, adc_rdptr,adc_wrptr,adc_length, flag;
int current_ptr1, delay_ptr1, current_ptr2, delay_ptr2;
int haveData, send_wrptr, send_rdptr, didRcv, end_ptr;
Xuint16 indata, outdata, instate1, outstate1, instate2, outstate2, temp, temp1, outmid;
WORD recvLength;

// INTERRUPT  SETUP

// using interrupts at this low level is not trivial
// somebody may find some logic in the following sequence

void setup_interrupts()
{

  // reset the interrupt controller

  // this disables the INTC irq output, so the INTC and CPU are "disconnected"
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  // disable all interrupt sources
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0); // disable all

  // ack all sources, to be sure none of them is pending due to circuit wakeup
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff); // ack all sources


  // initialize the drivers and interrupt handlers
  // here we are also supposed to reset the uart etc. TO DO

  // AUDIO AK4565 DRIVER SETUP

 // XIntc_InterruptVectorTable[XPAR_INTC_AK4565_INTERRUPT_INTR].Handler=audio_handler;
  XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_AK4565_INTERRUPT_INTR, audio_handler, NULL);


  audio_dac_bcnt_w=audio_dac_bcnt_r=0;

  // UART DRIVER & INTERRUPT SETUP


//  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler=uart_handler;
//XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_MYUART_INTERRUPT_INTR, uart_handler, NULL);


  // DONT't forget to initialize your driver data structures !!
  uart_no_interrupts = 0;

  // enable the interrupts

  // enable the CPU interrupts
   microblaze_enable_interrupts();

  // enable the INTC irq output
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR); // master enable

  // enable the INTC to receive interrupts from the AK4565 and UART
  //  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,
  //		    XPAR_AK4565_INTERRUPT_MASK | XPAR_MYUART_INTERRUPT_MASK);

  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AK4565_INTERRUPT_MASK);

  // enable the UART to generate interrupts
  // the audio driver always generate interrupts
  //XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);

}


// we prepare a square wave for audio output

/*
Xuint16 tri_wave[1024];
void make_triangle_wave()
{
  int i;
  for(i=0;i<512;i++) tri_wave[i]=tri_wave[1023-i]=(i-256)<<7;
}

// this functions ilustrates the user code
// it does 2 things:
// - plays a syrene sound
// - echoes all chars received from the uart

void make_some_noise()
{
  int phasel, phaser, dir, freq,i;
  char c;

  // clear_screen();

  make_triangle_wave();

  phasel=phaser=0; dir=0; freq=10000;
  while(1){

    if (audio_dac_bcnt_w - audio_dac_bcnt_r < 400)
      for(i=0;i<400;i++){
	audio_dac_buf[audio_dac_bcnt_w++ & 0x3FF] = tri_wave[(phaser>>11) & 0x3ff] +
	  (tri_wave[(phasel>>11) & 0x3FF] << 16);

	phasel+= 40000 - freq;
	phaser+=freq;

	if(dir){freq++; if(freq>30000) dir=0;} else
	  {freq--; if(freq<10000) dir=1;}
      }


    putnum(uart_no_interrupts); print("\n");

  }

}

*/

main()
{

  int i,a;
  BYTE sendBuffer[ETH_SEND_SIZE];
  BYTE *testBuffer = (BYTE*)DAC_START;
  adc_rdptr=0;
  adc_wrptr=0;
  haveData=0;
  adc_length=0;
  dac_rdptr=0;
  dac_wrptr=0;
  dac_length=0;
  delay_ptr1 = 0;
  current_ptr1 = DELAY_SIZE1; //K = DELAY_SIZE, BUFFER = DELAY_SIZE+1
  instate1=0;
  outstate1=0;
  delay_ptr2 = 0;
  current_ptr2 = DELAY_SIZE2; //K = DELAY_SIZE, BUFFER = DELAY_SIZE+1
  instate2=0;
  outstate2=0;
  temp=0;
  temp1 = 0;
  outmid=0;
  flag=0;
  send_rdptr=0;
  send_wrptr=0;
  didRcv=0;
  end_ptr=0;

  // Ok, the code is 6 times faster if cached
  microblaze_enable_icache();

  print("\r\n\r\n---------------Starting-------------\r\n\r\n");
 diag();

  // reset the NIC card
 if(!init_ethernet()){
    print("Ethernet NIC not detected");
    return;
  }

 

  //initialize the delay block to zeros
  for(i=0;i<DELAY_SIZE1;i++)
  {
    //unroll the loop
    XIo_Out16(DELAY_START1+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START1+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START1+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START1+(i<<1),0);
  }

  for(i=0;i<DELAY_SIZE2;i++)
  {
    //unroll the loop
    XIo_Out16(DELAY_START2+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START2+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START2+(i<<1),0);
    i++;
    XIo_Out16(DELAY_START2+(i<<1),0);
  }

  for(i = 0; i < ETH_SEND_SIZE; i++)
  {
	sendBuffer[i] = 'A';
  }

  sendBuffer[40] = 'H';
  sendBuffer[41] = 'e';
  sendBuffer[42] = 'l';
  sendBuffer[43] = 'l';
  sendBuffer[44] = 'o';


  print("Hello World!\r\n");

  // send(ETH_SEND_SIZE,(WORD *)sendBuffer);

  //for(a=0; a<640*480;a++)
   // XIo_Out8(VGA_START_ADDRESS+a, 0);

 // for(i=0;i<100;i++)
   // XIo_Out8(VGA_START_ADDRESS+i+i*640, 0xff);

  setup_interrupts();

 // for(i=0;i<100;i++)
  //  XIo_Out8(VGA_START_ADDRESS+100+i+i*640, 0xff);

  //  USER CODE
  //  print("Hello interrupts\n\r");

  //   make_some_noise();

  // for(i=0;i<100;i++)
   // XIo_Out8(VGA_START_ADDRESS+200+i+i*640, 0xff);



   while(1)
   {
     /*

   	if(flag==1)
	{
	  print("ADC buffer overflow\r\n");
	  microblaze_disable_interrupts();
	  flag=0;
	  microblaze_enable_interrupts();
	}
     //read from ADC FIFO
     //print("Reading from ADC FIFO\n\r");
     if(adc_length>0)
     {
       //data = audio_adc_buf[adc_rdptr];
       indata = XIo_In16(ADC_START + (adc_rdptr<<1));

       adc_rdptr++;

       microblaze_disable_interrupts();
	 adc_length--;
	 microblaze_enable_interrupts();

       haveData=1;
       //print("\n\rThe data is ");
       //putnum(data);
       //print("\n\rThe adc_rdptr is ");
       //putnum(adc_rdptr);
       if(adc_rdptr==SIZE)
	adc_rdptr=0;
     }
     else
       haveData=0;

     if(haveData==1)
     {

       //the first stage

       temp1 = outstate1+indata;

       if (temp1>>15 == 1)//sign extend temp1
       {
	 temp = ((temp1>>2) + 0xc000);
       }
       else
       {
	  temp = (temp1>>2);
       }
       //temp = ((outstate+indata)>>2)+((outstate+indata)>>1);
       instate1 = ~(temp)+1+indata;
       outmid = temp+outstate1;



       //delay block 1
       XIo_Out16(DELAY_START1 + (current_ptr1<<1), instate1);
       current_ptr1++;
       if(current_ptr1 == DELAY_SIZE1+1)
	 current_ptr1 = 0;


       outstate1 = XIo_In16(DELAY_START1 + (delay_ptr1<<1));
       delay_ptr1++;
       if(delay_ptr1 == DELAY_SIZE1+1)
	 delay_ptr1 = 0;



       //the second stage

       temp1 = outstate2+outmid;

       if (temp1>>15 == 1)//sign extend temp1
       {
	 temp = ((temp1>>2) + 0xc000) + ((temp1>>1) + 0x8000);
       }
       else
       {
	  temp = (temp1>>2) + (temp1>>1);
       }
       //temp = ((outstate+indata)>>2)+((outstate+indata)>>1);
       instate2 = ~(temp)+1+outmid;
       outdata = temp+outstate2;



       //delay block 2
       XIo_Out16(DELAY_START2 + (current_ptr2<<1), instate2);
       current_ptr2++;
       if(current_ptr2 == DELAY_SIZE2+1)
	 current_ptr2 = 0;


       outstate2 = XIo_In16(DELAY_START2 + (delay_ptr2<<1));
       delay_ptr2++;
       if(delay_ptr2 == DELAY_SIZE2+1)
	 delay_ptr2 = 0;






     //send the data to ethernet
     //print("Writing to DAC FIFO");

       //audio_dac_buf[dac_wrptr] = data;

       XIo_Out16(SEND_START + (send_wrptr<<1), outdata);


       send_wrptr++;





	if(send_wrptr==SIZE)
	{
		send_wrptr=0;
		//send((WORD)SIZE,(WORD *)SEND_START);

	}

     }//end if haveData


     */
	//get the data from the ethernet to the DAC's buffer
	//didRcv = 0;

	if (innic(ISR) & 0x01)
	{
		 receive(&recvLength,(BYTE *)(DAC_START+(dac_wrptr<<1)));
		 //if (recvLength!=SIZE)
		 //{
		 // print("ERROR!  RCV LENGTH NOT 80!\n\r");

		 //		 }
		 //else
		 // {
		   //didRcv = 1;
		 //print("I received something");
		 	//dac_wrptr+=80;
			dac_wrptr+=recvLength;
		 	if(dac_wrptr>=DAC_SIZE)
				dac_wrptr=dac_wrptr-DAC_SIZE;
		        microblaze_disable_interrupts();
	 		dac_length+=recvLength;
			microblaze_enable_interrupts();
			if((dac_wrptr==dac_rdptr))
			{
			  print("DAC Buffer Overflow\n\r");
			}
			//	}
		outnic(ISR,0xff);
	}

//if i did receive, I am done, since I already put it in DAC FIFO buffer
//otherwise, and if the DAC FIFO is empty, should give ISR some zeros should
//to send to DAC

	//write zeros to DAC
	/*
	if((didRcv==0)  && (dac_length==0))
	{

	  // print("Sending 0's to DAC\r\n");
	      
		microblaze_disable_interrupts();
		dac_length=80;
		microblaze_enable_interrupts();
		end_ptr = dac_wrptr+80;
		while(dac_wrptr<end_ptr)
		{
  		//for(i=0;i<SIZE;i++)

		    //unroll the loop
    		    XIo_Out16(DAC_START+(dac_wrptr<<1),0);
    	            dac_wrptr++;


    		   XIo_Out16(DAC_START+(dac_wrptr<<1),0);
                   dac_wrptr++;


    		   XIo_Out16(DAC_START+(dac_wrptr<<1),0);
		   dac_wrptr++;

    		   XIo_Out16(DAC_START+(dac_wrptr<<1),0);
		   dac_wrptr++;
  		}
		if(dac_wrptr==DAC_SIZE)
		     dac_wrptr=0;
	  
	}
	*/
	


	/*
	XIo_Out16(DAC_START + (dac_wrptr<<1), outdata);
        dac_wrptr++;
        microblaze_disable_interrupts();
	dac_length++;
       microblaze_enable_interrupts();
       if(dac_wrptr==DAC_SIZE)
       {
  		 dac_wrptr=0;
  	   //send((WORD)SIZE,(WORD *)DAC_START);
	}
*/



   }//end while
}//end main



