//#include "userio.h"
//#include "sound.h"
//#include "video.h"

#include "devices.h"
#include "video8.h"
#include "tetris_video8.h"

unsigned int hwrandom;

#define PSIZE 4
#define NP 7

typedef unsigned char piece [PSIZE][PSIZE];

piece p_templ[NP]={
{ {0,0,1,0},{0,0,1,0},{0,0,1,0},{0,0,1,0} }, // I
{ {0,0,0,0},{0,1,1,0},{0,1,1,0},{0,0,0,0} }, // O
{ {0,0,1,0},{0,1,1,0},{0,0,1,0},{0,0,0,0} }, // T
{ {0,1,0,0},{0,1,0,0},{0,1,1,0},{0,0,0,0} }, // L
{ {0,0,1,0},{0,0,1,0},{0,1,1,0},{0,0,0,0} }, // L mir
{ {0,1,0,0},{0,1,1,0},{0,0,1,0},{0,0,0,0} }, // Z
{ {0,0,1,0},{0,1,1,0},{0,1,0,0},{0,0,0,0} }  // Z mir
};

Xuint16 pieces_rgb[1+NP]={0x0,
	0xE060, 0x1C0C, 0x2701, // I O T
	0xBC6C, 0xA361, 0x1F0D, 0xAC64 // L L Z Z
};


#define TET_W 10
#define TET_H 29

#define KEY_RL 'a'
#define KEY_RR 'z'
#define KEY_SHL ','
#define KEY_SHR '.'
#define KEY_DROP ' '
#define KEY_START '|'


int kb_rl, kb_rr, kb_shl, kb_shr, kb_drop, kb_start;
// 0: inactive 1: active 2:same as 0
                
int x=0;

void rdkeys()
{
  int code;

  tetris_sound();

  draw_hex_num(40,22,audiodatain);

  while(uart_rx_bcnt_w > uart_rx_bcnt_r){
    code=uart_rx_buf[uart_rx_bcnt_r++ &0x3F];

    draw_char(40+x,50,code);
    x++;
    if(x>20)x=0;


    switch(code){
    case KEY_RL:  kb_rl = 1; break;
    case KEY_RR:  kb_rr = 1; break;
    case KEY_SHL: kb_shl = 1; break;
    case KEY_SHR: kb_shr = 1; break;
    case KEY_DROP: kb_drop = 1; break;
    case KEY_START: kb_start = 1; break;
    }
  }

}


unsigned char tet[TET_H][TET_W];

void display_tet()
{
	int x,y;
	for(y=0;y<TET_H;y++){
	  for(x=0;x<TET_W;x++) 
	    tetris_draw_square(x+1, y, pieces_rgb[tet[y][x]]);
	}  
}

#define BRICKCOLOR 0xE0FF
void init_screen()
{
	int x,y;

	for(y=0;y<30;y++){
		x=0; tetris_draw_brick(x, y, BRICKCOLOR);
		x=TET_W+1; tetris_draw_brick(x, y, BRICKCOLOR);
	}
	for(x=0;x<TET_W+1;x++){
		y=TET_H;
		tetris_draw_brick(x, y, BRICKCOLOR);
	}


	draw_string(45,10,"SCORE:");
	
	draw_string(45,12, "Hi score:");

	draw_string(45,14, "Shift left  : <");
	draw_string(45,15, "Shift right : >");
	draw_string(45,16, "Rotate left : a");
	draw_string(45,17, "Rotate right: z");
	draw_string(45,18, "Drop        : space");

	draw_string(45,20, "Start/Restart: |");
	
}

void rotate(piece *dst, piece *src, int rot)
{
  int x,y;
  for(x=0;x<PSIZE;x++)for(y=0;y<PSIZE;y++){
    switch(rot&3){
    case 0: (*dst)[x][y] = (*src)[x][y]; break;
    case 1: (*dst)[x][y] = (*src)[y][PSIZE - 1 - x]; break;
    case 2: (*dst)[x][y] = (*src)[PSIZE - 1 - x][PSIZE - 1 - y]; break;
    case 3: (*dst)[x][y] = (*src)[PSIZE - 1 - y][x]; break;
    }
  }
}

#define PLACE 0
#define ERASE 1
#define CHECK 2

int place(int ptype, int rot, int x, int y, int optype)
{
  piece t;
  int x1,y1,x2,y2;
  int inrng;
  
  rotate(&t, &p_templ[ptype-1], rot);
  
  for(x1=0;x1<PSIZE;x1++)for(y1=0;y1<PSIZE;y1++)
    if(t[x1][y1]){
      x2=x+x1; y2=y+y1;
      inrng = x2>=0 && x2<TET_W && y2>=0 && y2<TET_H;
      switch(optype){
      case PLACE:	if(inrng) tet[y2][x2] = ptype; break;
      case ERASE: if(inrng) tet[y2][x2] = 0; break;
      case CHECK: if(!inrng) return -1;
	if(tet[y2][x2]) return -1;
      }
    }
  return 0;
}

#define HINT_X 13
#define HINT_Y 5

void place_hint(int ptype, int rot)
{
  piece t;
  int x1,y1,x2,y2;
  Xuint32 rgb;
  
  rotate(&t, &p_templ[ptype-1], rot);
  rgb=pieces_rgb[ptype];
  
  for(x1=0;x1<PSIZE;x1++)for(y1=0;y1<PSIZE;y1++)
    tetris_draw_square(HINT_X + x1, HINT_Y + y1, t[x1][y1]?rgb:0x0 );

}


int collapse(int cy)
{
  int x,y, y2;
  int chk;
  int cleared=0;  

  for(y=cy+3; y>=cy; y--){
    if(y>=TET_H) continue;
    chk=1;
    for(x=0; x<TET_W; x++) if(!tet[y][x]) chk=0;
    while(chk){
	cleared++;	
	for(x=0; x<TET_W; x++){
      	for (y2=y; y2 > 0; y2--)  
 	   		tet[y2][x]=tet[y2-1][x];		
		tet[0][x]=0;
		if(!tet[y][x]) chk=0;
		}
	}
  }

  return cleared;
}


#define TICKS 10
#define BEEPLEN 500

main()
{

  int x,y;
  int cp, cr,cx, cy, cx1, cy1, cr1;
  int nxp, nxr;
  int score, hiscore;
  int i,t;
  int newp, tick;
  int key;

  microblaze_enable_icache();

  clear_screen();
  tetris_sound_init();

  setup_interrupts();

 
  

  init_screen();
  

  kb_rl = kb_rr = kb_shl = kb_shr = kb_start = 0;
  
  i=0;
  hiscore = score = 0;
  
  while(1){ // multigame loop
    
    if(score > hiscore ) hiscore = score;
    draw_dec_num(60,12,hiscore);
    
    while(kb_start != 1) rdkeys();
    kb_start=2;
    
    for(x=0;x<TET_W;x++)for(y=0;y<TET_H;y++) tet[y][x]=0;
    score=0;
    newp=1;
    nxp = 1 + (hwrandom % NP);
    nxr = (hwrandom >> 3) & 3;
    
    while(1){ // game loop
      
      //i++; if(i==10000) return;

      rdkeys();
      
      if(kb_start==1) break;
      
      draw_dec_num(60, 10, score);
      
      if(newp){
	cp = nxp;
	cr = nxr;
	hwrandom = hwrandom ^ ~cy1;
	nxp = 1 + (hwrandom % NP);
	nxr = (hwrandom >> 3) & 3;
	place_hint(nxp, nxr);
	
	cx = TET_W / 2;
	cy = 0;
	
	tick = TICKS;
	newp = 0;
	
	if(place(cp, cr, cx, cy, CHECK) == -1) break; // end game
	
      } else place(cp,cr,cx,cy, ERASE);
      
      // now the current piece is NOT in memory
      
      // keyboard
      
      if(kb_drop==1){
	cy1=cy+1;
	while(place(cp, cr, cx, cy1, CHECK) != -1) cy1++;
	cy = cy1 - 1;
	kb_drop=2;
	tick=1;
	sndbeeplen=BEEPLEN;
      } else {
	cx1=cx; cr1=cr;
	if(kb_rl==1) {cr1--; kb_rl=2; sndbeeplen=BEEPLEN;}
	if(kb_rr==1) {cr1++; kb_rr=2; sndbeeplen=BEEPLEN;}
	if(kb_shl==1) {cx1--; kb_shl=2; sndbeeplen=BEEPLEN;}
	if(kb_shr==1) {cx1++; kb_shr=2; sndbeeplen=BEEPLEN;}
	if(place(cp, cr1, cx1, cy, CHECK) != -1) {cx=cx1; cr=cr1;}
      }
      
      
      tick--;
      if(!tick){
	tick = TICKS;
	cy1 = cy + 1;
	if (place(cp, cr, cx, cy1, CHECK) == -1) newp = 1; else cy=cy1;
      }
      
      place(cp,cr,cx,cy, PLACE);
      if(newp) { t = collapse(cy); score+=t; if(t)sndchirplen = 4000; }
      
      display_tet();
    }
  }
  
}
