/* $Id: xuartlite_low_level_example.c,v 1.1 2002/05/02 20:31:53 moleres Exp $ */
/*****************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
*****************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xuartlite_low_level_example.c
*
* DESCRIPTION:
*
* This file contains a design example using the low-level driver functions
* and macros of the UART Lite driver (XUartLite).
*
* NOTES:
*
* None.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00b rpm  04/25/02 First release
*
******************************************************************************/

/***************************** Include Files *********************************/


#include "xuartlite_l.h"
#include "xparameters.h"

/************************** Constant Definitions *****************************/

/* the following constant controls the length of the buffers to be sent
 * and received with the UART, this constant must be 16 bytes or less so the
 * entire buffer will fit into the transmit and receive FIFOs of the UART
 */
#define TEST_BUFFER_SIZE 16


#define UARTLITE_BASEADDR       XPAR_MYUART_BASEADDR

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

/****************************************************************************
*
* FUNCTION:
*
* XUartLite_LowLevelExample
*
* DESCRIPTION:
*
* This function does a minimal test on the UART Lite device using the low-level
* driver macros and functions. This function sends data and expects to receive
* the data thru the UART. A physical loopback must be done by the user with the
* tranmit and receive signals of the UART.
*
* ARGUMENTS:
*
* None.
*
* RETURN VALUE:
*
* 0 if successful, otherwise -1.
*
* NOTES:
*
* None.
*
****************************************************************************/
int XUartLite_LowLevelExample()
{
    int Index;
    Xuint8 SendBuffer[TEST_BUFFER_SIZE];
    Xuint8 RecvBuffer[TEST_BUFFER_SIZE];


    /* Initialize the send buffer bytes with a pattern to send and the
     * the receive buffer bytes to zero
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        SendBuffer[Index] = 'A'+Index;
        RecvBuffer[Index] = 0;
    }

    /*
     * Send the entire transmit buffer.
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        XUartLite_SendByte(UARTLITE_BASEADDR, SendBuffer[Index]);
    }

    return 0;

    /*
     * Receive the entire buffer's worth. Note that the RecvByte function
     * blocks waiting for a character.
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        RecvBuffer[Index] = XUartLite_RecvByte(UARTLITE_BASEADDR);
    }

    /*
     * Check the receive buffer data against the send buffer and verify the
     * data was correctly received
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        if (SendBuffer[Index] != RecvBuffer[Index])
        {
            return -1;
        }
    }

    return 0;
}
